# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2020 Francis Deslauriers <francis.deslauriers@efficios.com>

import re
import sys


def main():
    expected = sys.argv[1]
    actual = sys.argv[2]
    sorted_expected = "".join(sorted(re.findall(r"\w+|\W+", expected.strip())))
    sorted_actual = "".join(sorted(re.findall(r"\w+|\W+", actual.strip())))

    if sorted_expected == sorted_actual:
        status = 0
    else:
        status = 1

    sys.exit(status)


if __name__ == "__main__":
    main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2022 EfficiOS, Inc.
#


# Decorator for unittest.TestCast sub-classes to run tests against CTF 1 and
# CTF 2 versions of the same traces.
#
# Replaces all test_* methods with a test_*_ctf_1 and a test_*_ctf_2 variant.
#
# For instance, it transforms this:
#
#   @test_all_ctf_versions
#   class MyTestCase(unittest.TestCase):
#       test_something(self):
#           pass
#
# into:
#
#   class MyTestcase(unittest.TestCase):
#       test_something_ctf_1(self):
#           pass
#
#       test_something_ctf_2(self):
#           pass
#
# The test methods are wrapped such that the self._ctf_version attribute is
# set to either 1 or 2 during the call to each method.
def test_all_ctf_versions(cls):
    for attr_name, attr_value in list(cls.__dict__.items()):
        if not attr_name.startswith("test_") or not callable(attr_value):
            continue

        for ctf_version in 1, 2:
            # Callable that wraps and replaces test methods in order to
            # temporarily set the _ctf_version attribute on the TestCase class.
            def set_ctf_version_wrapper_method(self, ctf_version, test_method):
                assert not hasattr(self, "_ctf_version")
                self._ctf_version = ctf_version

                try:
                    return test_method(self)
                finally:
                    assert hasattr(self, "_ctf_version")
                    del self._ctf_version

            def wrap_method(wrapper_method, ctf_version, original_method):
                return lambda self: wrapper_method(self, ctf_version, original_method)

            setattr(
                cls,
                "{}_ctf_{}".format(attr_name, ctf_version),
                wrap_method(set_ctf_version_wrapper_method, ctf_version, attr_value),
            )

        delattr(cls, attr_name)

    return cls



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import unittest

import bt2


class PackageTestCase(unittest.TestCase):
    def _assert_in_bt2(self, name):
        self.assertIn(name, dir(bt2))


_public_names = [
    "__version__",
    "_ArrayField",
    "_ArrayFieldClass",
    "_ArrayFieldClassConst",
    "_ArrayFieldConst",
    "_ArrayValueConst",
    "_BitArrayField",
    "_BitArrayFieldClass",
    "_BitArrayFieldClassConst",
    "_BitArrayFieldClassFlagConst",
    "_BitArrayFieldConst",
    "_BlobField",
    "_BlobFieldClass",
    "_BlobFieldClassConst",
    "_BlobFieldConst",
    "_BoolField",
    "_BoolFieldClass",
    "_BoolFieldClassConst",
    "_BoolFieldConst",
    "_BoolValueConst",
    "_ClockClass",
    "_ClockClassConst",
    "_ClockSnapshotConst",
    "_ComponentClassErrorCause",
    "_ComponentErrorCause",
    "_CurrentArrayElementFieldPathItem",
    "_CurrentOptionContentFieldPathItem",
    "_DiscardedEventsMessage",
    "_DiscardedEventsMessageConst",
    "_DiscardedPacketsMessage",
    "_DiscardedPacketsMessageConst",
    "_DoublePrecisionRealField",
    "_DoublePrecisionRealFieldClass",
    "_DoublePrecisionRealFieldClassConst",
    "_DoublePrecisionRealFieldConst",
    "_DynamicArrayField",
    "_DynamicArrayFieldClass",
    "_DynamicArrayFieldClassConst",
    "_DynamicArrayFieldClassWithLengthField",
    "_DynamicArrayFieldClassWithLengthFieldConst",
    "_DynamicArrayFieldConst",
    "_DynamicArrayFieldWithLengthField",
    "_DynamicArrayFieldWithLengthFieldConst",
    "_DynamicArrayWithLengthFieldFieldClass",
    "_DynamicArrayWithLengthFieldFieldClassConst",
    "_DynamicBlobField",
    "_DynamicBlobFieldClass",
    "_DynamicBlobFieldClassConst",
    "_DynamicBlobFieldClassWithLengthField",
    "_DynamicBlobFieldClassWithLengthFieldConst",
    "_DynamicBlobFieldConst",
    "_DynamicBlobFieldWithLengthField",
    "_DynamicBlobFieldWithLengthFieldConst",
    "_EnumerationField",
    "_EnumerationFieldClass",
    "_EnumerationFieldClassConst",
    "_EnumerationFieldConst",
    "_Error",
    "_ErrorCause",
    "_EventClass",
    "_EventClassConst",
    "_EventMessage",
    "_EventMessageConst",
    "_Field",
    "_FieldClass",
    "_FieldClassConst",
    "_FieldConst",
    "_FieldLocationConst",
    "_FieldPathConst",
    "_FieldPathItem",
    "_FilterComponentClassConst",
    "_FilterComponentConst",
    "_IncompleteUserClass",
    "_IndexFieldPathItem",
    "_InputPortConst",
    "_IntegerField",
    "_IntegerFieldClass",
    "_IntegerFieldClassConst",
    "_IntegerFieldConst",
    "_IntegerValue",
    "_IntegerValueConst",
    "_ListenerHandle",
    "_MapValueConst",
    "_MemoryError",
    "_MessageConst",
    "_MessageIteratorConfiguration",
    "_MessageIteratorErrorCause",
    "_MessageIteratorInactivityMessage",
    "_MessageIteratorInactivityMessageConst",
    "_OptionField",
    "_OptionFieldClass",
    "_OptionFieldClassConst",
    "_OptionFieldClassWithBoolSelectorField",
    "_OptionFieldClassWithBoolSelectorFieldConst",
    "_OptionFieldClassWithIntegerSelectorField",
    "_OptionFieldClassWithIntegerSelectorFieldConst",
    "_OptionFieldClassWithSelectorField",
    "_OptionFieldClassWithSelectorFieldConst",
    "_OptionFieldClassWithSignedIntegerSelectorField",
    "_OptionFieldClassWithSignedIntegerSelectorFieldConst",
    "_OptionFieldClassWithUnsignedIntegerSelectorField",
    "_OptionFieldClassWithUnsignedIntegerSelectorFieldConst",
    "_OptionFieldConst",
    "_OptionFieldWithBoolSelectorField",
    "_OptionFieldWithBoolSelectorFieldConst",
    "_OptionFieldWithSignedIntegerSelectorField",
    "_OptionFieldWithSignedIntegerSelectorFieldConst",
    "_OptionFieldWithUnsignedIntegerSelectorField",
    "_OptionFieldWithUnsignedIntegerSelectorFieldConst",
    "_OptionWithBoolSelectorFieldClass",
    "_OptionWithBoolSelectorFieldClassConst",
    "_OptionWithIntegerSelectorFieldClass",
    "_OptionWithIntegerSelectorFieldClassConst",
    "_OptionWithSelectorFieldClass",
    "_OptionWithSelectorFieldClassConst",
    "_OptionWithSignedIntegerSelectorFieldClass",
    "_OptionWithSignedIntegerSelectorFieldClassConst",
    "_OptionWithUnsignedIntegerSelectorFieldClass",
    "_OptionWithUnsignedIntegerSelectorFieldClassConst",
    "_OutputPortConst",
    "_OverflowError",
    "_PacketBeginningMessage",
    "_PacketBeginningMessageConst",
    "_PacketEndMessage",
    "_PacketEndMessageConst",
    "_PluginSet",
    "_PrivateQueryExecutor",
    "_RealField",
    "_RealFieldClass",
    "_RealFieldClassConst",
    "_RealFieldConst",
    "_RealValueConst",
    "_SignedEnumerationField",
    "_SignedEnumerationFieldClass",
    "_SignedEnumerationFieldClassConst",
    "_SignedEnumerationFieldClassMappingConst",
    "_SignedEnumerationFieldConst",
    "_SignedIntegerField",
    "_SignedIntegerFieldClass",
    "_SignedIntegerFieldClassConst",
    "_SignedIntegerFieldConst",
    "_SignedIntegerRangeConst",
    "_SignedIntegerRangeSetConst",
    "_SignedIntegerValueConst",
    "_SinglePrecisionRealField",
    "_SinglePrecisionRealFieldClass",
    "_SinglePrecisionRealFieldClassConst",
    "_SinglePrecisionRealFieldConst",
    "_SinkComponentClassConst",
    "_SinkComponentConst",
    "_SourceComponentClassConst",
    "_SourceComponentConst",
    "_StaticArrayField",
    "_StaticArrayFieldClass",
    "_StaticArrayFieldClassConst",
    "_StaticArrayFieldConst",
    "_StaticBlobField",
    "_StaticBlobFieldClass",
    "_StaticBlobFieldClassConst",
    "_StaticBlobFieldConst",
    "_Stream",
    "_StreamBeginningMessage",
    "_StreamBeginningMessageConst",
    "_StreamClass",
    "_StreamClassConst",
    "_StreamConst",
    "_StreamEndMessage",
    "_StreamEndMessageConst",
    "_StringField",
    "_StringFieldClass",
    "_StringFieldClassConst",
    "_StringFieldConst",
    "_StringValueConst",
    "_StructureField",
    "_StructureFieldClass",
    "_StructureFieldClassConst",
    "_StructureFieldClassMember",
    "_StructureFieldClassMemberConst",
    "_StructureFieldConst",
    "_Trace",
    "_TraceClass",
    "_TraceClassConst",
    "_TraceConst",
    "_UnixEpochClockOrigin",
    "_UnknownClockOrigin",
    "_UnknownClockSnapshot",
    "_UnsignedEnumerationField",
    "_UnsignedEnumerationFieldClass",
    "_UnsignedEnumerationFieldClassConst",
    "_UnsignedEnumerationFieldClassMappingConst",
    "_UnsignedEnumerationFieldConst",
    "_UnsignedIntegerField",
    "_UnsignedIntegerFieldClass",
    "_UnsignedIntegerFieldClassConst",
    "_UnsignedIntegerFieldConst",
    "_UnsignedIntegerRangeConst",
    "_UnsignedIntegerRangeSetConst",
    "_UnsignedIntegerValueConst",
    "_UserComponentInputPort",
    "_UserComponentInputPortMessageIterator",
    "_UserComponentOutputPort",
    "_UserFilterComponent",
    "_UserFilterComponentConfiguration",
    "_UserMessageIterator",
    "_UserSinkComponent",
    "_UserSinkComponentConfiguration",
    "_UserSourceComponent",
    "_UserSourceComponentConfiguration",
    "_Value",
    "_ValueConst",
    "_VariantField",
    "_VariantField",
    "_VariantFieldClass",
    "_VariantFieldClassConst",
    "_VariantFieldClassOption",
    "_VariantFieldClassOptionConst",
    "_VariantFieldClassWithIntegerSelector",
    "_VariantFieldClassWithIntegerSelectorConst",
    "_VariantFieldClassWithIntegerSelectorField",
    "_VariantFieldClassWithIntegerSelectorFieldConst",
    "_VariantFieldClassWithIntegerSelectorFieldOption",
    "_VariantFieldClassWithIntegerSelectorFieldOptionConst",
    "_VariantFieldClassWithoutSelector",
    "_VariantFieldClassWithoutSelectorConst",
    "_VariantFieldClassWithoutSelectorField",
    "_VariantFieldClassWithoutSelectorFieldConst",
    "_VariantFieldClassWithSignedIntegerSelector",
    "_VariantFieldClassWithSignedIntegerSelectorConst",
    "_VariantFieldClassWithSignedIntegerSelectorField",
    "_VariantFieldClassWithSignedIntegerSelectorFieldConst",
    "_VariantFieldClassWithSignedIntegerSelectorFieldOption",
    "_VariantFieldClassWithSignedIntegerSelectorFieldOptionConst",
    "_VariantFieldClassWithUnsignedIntegerSelector",
    "_VariantFieldClassWithUnsignedIntegerSelectorConst",
    "_VariantFieldClassWithUnsignedIntegerSelectorField",
    "_VariantFieldClassWithUnsignedIntegerSelectorFieldConst",
    "_VariantFieldClassWithUnsignedIntegerSelectorFieldOption",
    "_VariantFieldClassWithUnsignedIntegerSelectorFieldOptionConst",
    "_VariantFieldConst",
    "_VariantFieldConst",
    "_VariantFieldWithSignedIntegerSelectorField",
    "_VariantFieldWithSignedIntegerSelectorFieldConst",
    "_VariantFieldWithUnsignedIntegerSelectorField",
    "_VariantFieldWithUnsignedIntegerSelectorFieldConst",
    "ArrayValue",
    "AutoSourceComponentSpec",
    "BoolValue",
    "ClockClassOffset",
    "ClockOffset",
    "ClockOrigin",
    "ComponentClassType",
    "ComponentDescriptor",
    "ComponentSpec",
    "create_value",
    "EventClassLogLevel",
    "FieldLocationScope",
    "FieldPathScope",
    "find_plugin",
    "find_plugins",
    "find_plugins_in_path",
    "get_global_logging_level",
    "get_greatest_operative_mip_version",
    "get_maximal_mip_version",
    "get_minimal_logging_level",
    "Graph",
    "IntegerDisplayBase",
    "Interrupter",
    "LoggingLevel",
    "MapValue",
    "plugin_component_class",
    "QueryExecutor",
    "RealValue",
    "register_plugin",
    "set_global_logging_level",
    "SignedIntegerRange",
    "SignedIntegerRangeSet",
    "SignedIntegerValue",
    "Stop",
    "StringValue",
    "TraceCollectionMessageIterator",
    "TryAgain",
    "unix_epoch_clock_origin",
    "unknown_clock_origin",
    "UnknownObject",
    "UnsignedIntegerRange",
    "UnsignedIntegerRangeSet",
    "UnsignedIntegerValue",
]

for name in _public_names:

    def test_func(self):
        self._assert_in_bt2(name)

    setattr(PackageTestCase, "test_has_" + name, test_func)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

# pyright: strict, reportPrivateUsage=false

import uuid
import typing
import unittest

import bt2
from bt2 import value as bt2_value
from bt2 import clock_class as bt2_clock_class
from utils import (
    TestOutputPortMessageIterator,
    run_in_component_init,
    get_const_event_message,
)
from test_all_mip_versions import test_all_mip_versions


class ClockOffsetTestCase(unittest.TestCase):
    def test_create_default(self):
        cco = bt2.ClockOffset()
        self.assertEqual(cco.seconds, 0)
        self.assertEqual(cco.cycles, 0)

    def test_create(self):
        cco = bt2.ClockOffset(23, 4871232)
        self.assertEqual(cco.seconds, 23)
        self.assertEqual(cco.cycles, 4871232)

    def test_create_kwargs(self):
        cco = bt2.ClockOffset(seconds=23, cycles=4871232)
        self.assertEqual(cco.seconds, 23)
        self.assertEqual(cco.cycles, 4871232)

    def test_create_invalid_seconds(self):
        with self.assertRaises(TypeError):
            bt2.ClockOffset("hello", 4871232)  # type: ignore

    def test_create_invalid_cycles(self):
        with self.assertRaises(TypeError):
            bt2.ClockOffset(23, "hello")  # type: ignore

    def test_eq(self):
        cco1 = bt2.ClockOffset(23, 42)
        cco2 = bt2.ClockOffset(23, 42)
        self.assertEqual(cco1, cco2)

    def test_ne_seconds(self):
        cco1 = bt2.ClockOffset(23, 42)
        cco2 = bt2.ClockOffset(24, 42)
        self.assertNotEqual(cco1, cco2)

    def test_ne_cycles(self):
        cco1 = bt2.ClockOffset(23, 42)
        cco2 = bt2.ClockOffset(23, 43)
        self.assertNotEqual(cco1, cco2)

    def test_eq_invalid(self):
        self.assertFalse(bt2.ClockOffset() == 23)


RetT = typing.TypeVar("RetT")


@test_all_mip_versions
class ClockClassTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Big hack to make `_mip_version` known to static type checkers in a
        # Python 3.4 compatible way.
        cls._mip_version = -1
        del cls._mip_version

    def run_in_component_init(self, f: typing.Callable[[bt2._UserSinkComponent], RetT]):
        return run_in_component_init(self._mip_version, f)

    def assertRaisesInComponentInit(
        self,
        expected_exc_type: typing.Type[Exception],
        user_code: typing.Callable[[bt2._UserSinkComponent], RetT],
        expected_str: typing.Optional[str] = None,
    ):
        def f(comp_self: bt2._UserSinkComponent):
            try:
                user_code(comp_self)
            except Exception as exc:
                return exc

        exc = self.run_in_component_init(f)
        self.assertIsNotNone(exc)
        self.assertEqual(type(exc), expected_exc_type)

        if expected_str is not None:
            self.assertIn(expected_str, str(exc))

    def test_create_default(self):
        cc = run_in_component_init(
            self._mip_version, lambda comp_self: comp_self._create_clock_class()
        )

        self.assertIsNone(cc.name)
        self.assertEqual(cc.frequency, 1000000000)
        self.assertIsNone(cc.description)

        if self._mip_version == 0:
            self.assertEqual(cc.precision, 0)
        else:
            self.assertIsNone(cc.precision)
            self.assertIsNone(cc.accuracy)

        self.assertEqual(cc.offset, bt2.ClockOffset())
        self.assertIs(cc.origin, bt2.unix_epoch_clock_origin)

        if self._mip_version == 0:
            self.assertIsNone(cc.uuid)
        else:
            self.assertIsNone(cc.namespace)
            self.assertIsNone(cc.name)
            self.assertIsNone(cc.uid)

        self.assertEqual(len(cc.user_attributes), 0)

    def test_create_namespace(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(namespace="the_clock")

        if self._mip_version == 0:
            self.assertRaisesInComponentInit(
                ValueError,
                f,
                "Clock class namespace is only available with MIP version ≥ 1 (currently 0)",
            )
        else:
            self.assertEqual(self.run_in_component_init(f).namespace, "the_clock")

    def test_create_invalid_namespace(self):
        if self._mip_version < 1:
            self.skipTest("requires MIP >= 1")

        def f(comp_self: bt2._UserSinkComponent):
            comp_self._create_clock_class(namespace=23)  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f, "'int' is not a 'str' object")

    def test_create_name(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(name="the_clock")

        cc = self.run_in_component_init(f)
        self.assertEqual(cc.name, "the_clock")

    def test_create_invalid_name(self):
        def f(comp_self: bt2._UserSinkComponent):
            comp_self._create_clock_class(name=23)  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_uid(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(uid="the_clock")

        if self._mip_version == 0:
            self.assertRaisesInComponentInit(
                ValueError,
                f,
                "Clock class UID is only available with MIP version ≥ 1 (currently 0)",
            )
        else:
            self.assertEqual(self.run_in_component_init(f).uid, "the_clock")

    def test_create_invalid_uid(self):
        if self._mip_version < 1:
            self.skipTest("requires MIP >= 1")

        def f(comp_self: bt2._UserSinkComponent):
            comp_self._create_clock_class(uid=23)  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f, "'int' is not a 'str' object")

    def test_create_description(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(description="hi people")

        cc = self.run_in_component_init(f)
        self.assertEqual(cc.description, "hi people")

    def test_create_invalid_description(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(description=23)  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_frequency(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(frequency=987654321)

        cc = self.run_in_component_init(f)
        self.assertEqual(cc.frequency, 987654321)

    def test_create_invalid_frequency(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(frequency="lel")  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_precision(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(precision=12)

        self.assertEqual(self.run_in_component_init(f).precision, 12)

    def test_create_accuracy(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(accuracy=12)

        if self._mip_version == 0:
            self.assertRaisesInComponentInit(
                ValueError,
                f,
                "Clock class accuracy is only available with MIP version ≥ 1 (currently 0)",
            )
        else:
            self.assertEqual(self.run_in_component_init(f).accuracy, 12)

    def test_create_invalid_precision(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(precision="lel")  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_offset(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(offset=bt2.ClockOffset(12, 56))

        cc = self.run_in_component_init(f)
        self.assertEqual(cc.offset, bt2.ClockOffset(12, 56))

    def test_create_invalid_offset(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(offset=object())  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_origin_is_unix_epoch_true(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(origin_is_unix_epoch=True)

        cc = self.run_in_component_init(f)
        self.assertIs(cc.origin, bt2.unix_epoch_clock_origin)

    def test_create_origin_is_unix_epoch_false(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(origin_is_unix_epoch=False)

        cc = self.run_in_component_init(f)
        self.assertIs(cc.origin, bt2.unknown_clock_origin)

    def test_create_invalid_origin_is_unix_epoch(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(origin_is_unix_epoch=23)  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_unix_epoch_origin(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(origin=bt2.unix_epoch_clock_origin)

        cc = self.run_in_component_init(f)

        self.assertIs(cc.origin, bt2.unix_epoch_clock_origin)

    def test_create_custom_origin(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(
                origin=bt2.ClockOrigin("ns", "name", "uid")
            )

        if self._mip_version == 0:
            self.assertRaisesInComponentInit(
                ValueError,
                f,
                "Custom clock class origin is only available with MIP version ≥ 1 (currently 0)",
            )
        else:
            origin = self.run_in_component_init(f).origin

            assert type(origin) is bt2.ClockOrigin
            self.assertEqual(origin.namespace, "ns")
            self.assertEqual(origin.name, "name")
            self.assertEqual(origin.uid, "uid")

    def test_create_unknown_origin(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(origin=bt2.unknown_clock_origin)

        cc = self.run_in_component_init(f)

        self.assertIs(cc.origin, bt2.unknown_clock_origin)

    def test_create_invalid_origin(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(origin=23)  # type: ignore

        self.assertRaisesInComponentInit(
            TypeError,
            f,
            "'int' is not a '<class 'bt2.clock_class.ClockOrigin'>' object",
        )

    def test_create_multiple_origin(self):
        cases = (
            (True, bt2.ClockOrigin("ns", "name", "uid")),
            (False, bt2.ClockOrigin("ns", "name", "uid")),
        )

        for case in cases:
            origin_is_unix_epoch, origin = case

            class Foo:
                def __init__(
                    self,
                    origin_is_unix_epoch: typing.Optional[bool],
                    origin: typing.Optional[bt2.ClockOrigin],
                ):
                    self.origin_is_unix_epoch = origin_is_unix_epoch
                    self.origin = origin

                def __call__(self, comp_self: bt2._UserSinkComponent):
                    return comp_self._create_clock_class(
                        origin_is_unix_epoch=self.origin_is_unix_epoch,
                        origin=self.origin,
                    )

            self.assertRaisesInComponentInit(
                ValueError,
                Foo(origin_is_unix_epoch, origin),
                "only one of `origin_is_unix_epoch` and `origin` can be set",
            )

    def test_cycles_to_ns_from_origin(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(
                frequency=10**8, origin_is_unix_epoch=True
            )

        cc = self.run_in_component_init(f)
        self.assertEqual(cc.cycles_to_ns_from_origin(112), 1120)

    def test_cycles_to_ns_from_origin_overflow(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(frequency=1000)

        cc = self.run_in_component_init(f)
        with self.assertRaises(bt2._OverflowError):
            cc.cycles_to_ns_from_origin(2**63)

    def test_create_uuid(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(
                uuid=uuid.UUID("b43372c32ef0be28444dfc1c5cdafd33")
            )

        if self._mip_version == 0:
            self.assertEqual(
                self.run_in_component_init(f).uuid,
                uuid.UUID("b43372c32ef0be28444dfc1c5cdafd33"),
            )
        else:
            self.assertRaisesInComponentInit(
                ValueError,
                f,
                "Clock class UUID is only available with MIP version 0 (currently 1)",
            )

    def test_create_invalid_uuid(self):
        if self._mip_version != 0:
            self.skipTest("requires MIP == 0")

        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(uuid=23)  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_user_attributes(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(user_attributes={"salut": 23})

        cc = self.run_in_component_init(f)
        self.assertEqual(cc.user_attributes, {"salut": 23})
        self.assertIs(type(cc.user_attributes), bt2_value.MapValue)

    def test_const_user_attributes(self):
        cc = get_const_event_message().default_clock_snapshot.clock_class
        self.assertEqual(cc.user_attributes, {"a-clock-class-attribute": 1})
        self.assertIs(type(cc.user_attributes), bt2._MapValueConst)

    def test_create_invalid_user_attributes(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(user_attributes=object())  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)

    def test_create_invalid_user_attributes_value_type(self):
        def f(comp_self: bt2._UserSinkComponent):
            return comp_self._create_clock_class(user_attributes=23)  # type: ignore

        self.assertRaisesInComponentInit(TypeError, f)


class ClockSnapshotTestCase(unittest.TestCase):
    def run_in_component_init(self, f: typing.Callable[[bt2._UserSinkComponent], RetT]):
        return run_in_component_init(0, f)

    def setUp(self):
        def f(comp_self: bt2._UserSinkComponent):
            cc = comp_self._create_clock_class(
                1000, "my_cc", offset=bt2.ClockOffset(45, 354)
            )
            tc = comp_self._create_trace_class()

            return (cc, tc)

        _cc, _tc = self.run_in_component_init(f)
        _trace = _tc()
        _sc = _tc.create_stream_class(default_clock_class=_cc)
        _ec = _sc.create_event_class(name="salut")
        _stream = _trace.create_stream(_sc)
        self._stream = _stream
        self._ec = _ec
        self._cc = _cc

        class MyIter(bt2._UserMessageIterator):
            def __init__(
                self,
                config: bt2._MessageIteratorConfiguration,
                self_port_output: bt2._UserComponentOutputPort,
            ):
                self._at = 0

            def __next__(self):
                if self._at == 0:
                    notif = self._create_stream_beginning_message(_stream)
                elif self._at == 1:
                    notif = self._create_event_message(_ec, _stream, 123)
                elif self._at == 2:
                    notif = self._create_event_message(_ec, _stream, 2**63)
                elif self._at == 3:
                    notif = self._create_stream_end_message(_stream)
                else:
                    raise bt2.Stop

                self._at += 1
                return notif

        class MySrc(bt2._UserSourceComponent, message_iterator_class=MyIter):
            def __init__(
                self,
                config: bt2._UserSourceComponentConfiguration,
                params: bt2._MapValueConst,
                obj: object,
            ):
                self._add_output_port("out")

        self._graph = bt2.Graph()
        self._src_comp = self._graph.add_component(MySrc, "my_source")
        self._msg_iter = TestOutputPortMessageIterator(
            self._graph, self._src_comp.output_ports["out"]
        )

        for i, msg in enumerate(self._msg_iter):
            if i == 1:
                assert type(msg) is bt2._EventMessageConst
                self._msg = msg
            elif i == 2:
                assert type(msg) is bt2._EventMessageConst
                self._msg_clock_overflow = msg
                break

    def tearDown(self):
        del self._cc
        del self._msg

    def test_create_default(self):
        self.assertEqual(
            self._msg.default_clock_snapshot.clock_class.addr, self._cc.addr
        )
        self.assertEqual(self._msg.default_clock_snapshot.value, 123)

    def test_clock_class(self):
        cc = self._msg.default_clock_snapshot.clock_class
        self.assertEqual(cc.addr, self._cc.addr)
        self.assertIs(type(cc), bt2_clock_class._ClockClassConst)

    def test_ns_from_origin(self):
        s_from_origin = 45 + ((354 + 123) / 1000)
        ns_from_origin = int(s_from_origin * 1e9)
        self.assertEqual(
            self._msg.default_clock_snapshot.ns_from_origin, ns_from_origin
        )

    def test_ns_from_origin_overflow(self):
        with self.assertRaises(bt2._OverflowError):
            self._msg_clock_overflow.default_clock_snapshot.ns_from_origin

    def test_eq_int(self):
        self.assertEqual(self._msg.default_clock_snapshot, 123)

    def test_eq_invalid(self):
        self.assertFalse(self._msg.default_clock_snapshot == 23)

    def test_comparison(self):
        self.assertTrue(self._msg.default_clock_snapshot > 100)
        self.assertFalse(self._msg.default_clock_snapshot > 200)

        self.assertTrue(self._msg.default_clock_snapshot >= 123)
        self.assertFalse(self._msg.default_clock_snapshot >= 200)

        self.assertTrue(self._msg.default_clock_snapshot < 200)
        self.assertFalse(self._msg.default_clock_snapshot < 100)

        self.assertTrue(self._msg.default_clock_snapshot <= 123)
        self.assertFalse(self._msg.default_clock_snapshot <= 100)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import bt2


def to_string(p):
    # Print BT values in a predictable way (the order of map entries) and with
    # additional information (u suffix to differentiate unsigned integers from
    # signed integers).

    if type(p) is bt2._ArrayValueConst:
        s = "[{}]".format(", ".join([to_string(x) for x in p]))
    elif type(p) is bt2._MapValueConst:
        s = "{{{}}}".format(
            ", ".join([k + "=" + to_string(p[k]) for k in sorted(p.keys())])
        )
    elif type(p) is bt2._UnsignedIntegerValueConst:
        s = str(p) + "u"
    elif type(p) is bt2._RealValueConst:
        s = "{:.7f}".format(float(p))
    elif (
        type(p)
        in (
            bt2._StringValueConst,
            bt2._SignedIntegerValueConst,
            bt2._BoolValueConst,
        )
        or p is None
    ):
        s = str(p)

    else:
        raise TypeError("Unexpected type", type(p))

    return s



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import unittest

import bt2


class InterrupterTestCase(unittest.TestCase):
    def setUp(self):
        self._interrupter = bt2.Interrupter()

    def test_create(self):
        self.assertFalse(self._interrupter.is_set)

    def test_is_set(self):
        self.assertFalse(self._interrupter.is_set)

    def test_bool(self):
        self.assertFalse(self._interrupter)
        self._interrupter.set()
        self.assertTrue(self._interrupter)

    def test_set(self):
        self.assertFalse(self._interrupter)
        self._interrupter.set()
        self.assertTrue(self._interrupter)

    def test_reset(self):
        self._interrupter.set()
        self.assertTrue(self._interrupter)
        self._interrupter.reset()
        self.assertFalse(self._interrupter)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 Simon Marchi <simon.marchi@efficios.com>
#

import os
import unittest

import bt2
from test_all_ctf_versions import test_all_ctf_versions

query_object = "babeltrace.support-info"


@test_all_ctf_versions
class QuerySupportInfoTestCase(unittest.TestCase):
    def setUp(self):
        ctf = bt2.find_plugin("ctf")
        assert ctf
        self._fs = ctf.source_component_classes["fs"]

    def test_non_map_params(self):
        with self.assertRaisesRegex(
            bt2._Error, "Error validating parameters: top-level is not a map value"
        ):
            bt2.QueryExecutor(self._fs, query_object).query()

    def test_support_info_with_uuid(self):
        # Test that the right group is reported for each trace.

        session_rotation_trace_path = os.path.join(
            os.environ["BT_CTF_TRACES_PATH"],
            str(self._ctf_version),
            "succeed",
            "session-rotation",
        )

        trace_10352_1 = os.path.join(
            session_rotation_trace_path,
            "a",
            "1",
            "ust",
            "pid",
            "10352",
        )
        trace_10353_1 = os.path.join(
            session_rotation_trace_path,
            "a",
            "1",
            "ust",
            "pid",
            "10353",
        )
        trace_10352_2 = os.path.join(
            session_rotation_trace_path,
            "a",
            "2",
            "ust",
            "pid",
            "10352",
        )
        trace_10353_2 = os.path.join(
            session_rotation_trace_path,
            "a",
            "2",
            "ust",
            "pid",
            "10353",
        )
        trace_10352_3 = os.path.join(
            session_rotation_trace_path,
            "3",
            "ust",
            "pid",
            "10352",
        )
        trace_10353_3 = os.path.join(
            session_rotation_trace_path,
            "3",
            "ust",
            "pid",
            "10353",
        )

        def do_one_query(input, expected_group):
            qe = bt2.QueryExecutor(
                self._fs,
                query_object,
                {"input": input, "type": "directory"},
            )

            result = qe.query()
            self.assertEqual(result["group"], expected_group)

        do_one_query(
            trace_10352_1,
            "namespace: lttng.org,2009, name: , uid: 21cdfa5e-9a64-490a-832c-53aca6c101ba",
        )
        do_one_query(
            trace_10352_2,
            "namespace: lttng.org,2009, name: , uid: 21cdfa5e-9a64-490a-832c-53aca6c101ba",
        )
        do_one_query(
            trace_10352_3,
            "namespace: lttng.org,2009, name: , uid: 21cdfa5e-9a64-490a-832c-53aca6c101ba",
        )
        do_one_query(
            trace_10353_1,
            "namespace: lttng.org,2009, name: , uid: 83656eb1-b131-40e7-9666-c04ae279b58c",
        )
        do_one_query(
            trace_10353_2,
            "namespace: lttng.org,2009, name: , uid: 83656eb1-b131-40e7-9666-c04ae279b58c",
        )
        do_one_query(
            trace_10353_3,
            "namespace: lttng.org,2009, name: , uid: 83656eb1-b131-40e7-9666-c04ae279b58c",
        )


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2023 EfficiOS Inc.
#
# pyright: strict, reportTypeCommentUsage=false

import os
import string
import argparse

import normand
import moultipart


def _make_ctf_1_metadata(payload_fc: str):
    if "@" in payload_fc:
        payload_fc = payload_fc.replace("@", "root")
    else:
        payload_fc += " root"

    return string.Template(
        """\
/* CTF 1.8 */

trace {
    major = 1;
    minor = 8;
    byte_order = le;
};

typealias integer { size = 8; } := u8;
typealias integer { size = 16; } := u16;
typealias integer { size = 32; } := u32;
typealias integer { size = 64; } := u64;
typealias integer { size = 8; byte_order = le; } := u8le;
typealias integer { size = 16; byte_order = le; } := u16le;
typealias integer { size = 32; byte_order = le; } := u32le;
typealias integer { size = 64; byte_order = le; } := u64le;
typealias integer { size = 8; byte_order = be; } := u8be;
typealias integer { size = 16; byte_order = be; } := u16be;
typealias integer { size = 32; byte_order = be; } := u32be;
typealias integer { size = 64; byte_order = be; } := u64be;
typealias integer { signed = true; size = 8; } := i8;
typealias integer { signed = true; size = 16; } := i16;
typealias integer { signed = true; size = 32; } := i32;
typealias integer { signed = true; size = 64; } := i64;
typealias integer { signed = true; size = 8; byte_order = le; } := i8le;
typealias integer { signed = true; size = 16; byte_order = le; } := i16le;
typealias integer { signed = true; size = 32; byte_order = le; } := i32le;
typealias integer { signed = true; size = 64; byte_order = le; } := i64le;
typealias integer { signed = true; size = 8; byte_order = be; } := i8be;
typealias integer { signed = true; size = 16; byte_order = be; } := i16be;
typealias integer { signed = true; size = 32; byte_order = be; } := i32be;
typealias integer { signed = true; size = 64; byte_order = be; } := i64be;
typealias floating_point { exp_dig = 8; mant_dig = 24; } := flt32;
typealias floating_point { exp_dig = 11; mant_dig = 53; } := flt64;
typealias floating_point { exp_dig = 8; mant_dig = 24; byte_order = le; } := flt32le;
typealias floating_point { exp_dig = 11; mant_dig = 53; byte_order = le; } := flt64le;
typealias floating_point { exp_dig = 8; mant_dig = 24; byte_order = be; } := flt32be;
typealias floating_point { exp_dig = 11; mant_dig = 53; byte_order = be; } := flt64be;
typealias string { encoding = UTF8; } := nt_str;

event {
    name = the_event;
    fields := struct {
        ${payload_fc};
    };
};
"""
    ).substitute(payload_fc=payload_fc)


def _make_ctf_2_metadata(payload_fc: str):
    return string.Template(
        """\
\x1e{
  "type": "preamble",
  "version": "2"
}
\x1e{
  "type": "field-class-alias",
  "name": "u8le",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 8,
    "byte-order": "little-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u16le",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 16,
    "byte-order": "little-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u32le",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 32,
    "byte-order": "little-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u64le",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 64,
    "byte-order": "little-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u8",
  "field-class": "u8le"
}
\x1e{
  "type": "field-class-alias",
  "name": "u16",
  "field-class": "u16le"
}
\x1e{
  "type": "field-class-alias",
  "name": "u32",
  "field-class": "u32le"
}
\x1e{
  "type": "field-class-alias",
  "name": "u64",
  "field-class": "u64le"
}
\x1e{
  "type": "field-class-alias",
  "name": "u8be",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 8,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u16be",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 16,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u32be",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 32,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u64be",
  "field-class": {
    "type": "fixed-length-unsigned-integer",
    "length": 64,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "i8le",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 8,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "i16le",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 16,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "i32le",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 32,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "i64le",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 64,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "i8be",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 8,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "i16be",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 16,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "i32be",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 32,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "u64be",
  "field-class": {
    "type": "fixed-length-signed-integer",
    "length": 64,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "flt32le",
  "field-class": {
    "type": "fixed-length-floating-point-number",
    "length": 32,
    "byte-order": "little-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "flt64le",
  "field-class": {
    "type": "fixed-length-floating-point-number",
    "length": 64,
    "byte-order": "little-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "flt32",
  "field-class": "flt32le"
}
\x1e{
  "type": "field-class-alias",
  "name": "flt64",
  "field-class": "flt64le"
}
\x1e{
  "type": "field-class-alias",
  "name": "flt32be",
  "field-class": {
    "type": "fixed-length-floating-point-number",
    "length": 32,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "flt64be",
  "field-class": {
    "type": "fixed-length-floating-point-number",
    "length": 64,
    "byte-order": "big-endian",
    "alignment": 8
  }
}
\x1e{
  "type": "field-class-alias",
  "name": "nt-str",
  "field-class": {
    "type": "null-terminated-string"
  }
}
\x1e{
  "type": "data-stream-class"
}
\x1e{
  "type": "event-record-class",
  "payload-field-class": {
    "type": "structure",
    "member-field-classes": [
      {
        "name": "root",
        "field-class": ${payload_fc}
      }
    ]
  }
}
"""
    ).substitute(payload_fc=payload_fc)


def _make_ctf_metadata(payload_fc: str):
    if payload_fc.startswith("{") or payload_fc.startswith('"'):
        # CTF 2
        return _make_ctf_2_metadata(payload_fc)
    else:
        # Assume CTF 1.8
        return _make_ctf_1_metadata(payload_fc)


def _make_ctf_1_data(normand_text: str):
    # Default to little-endian because that's also the default in
    # _make_ctf_1_metadata() and _make_ctf_2_metadata() above.
    return normand.parse("!le\n" + normand_text).data


def _create_files_from_mp(mp_path: str, output_dir: str):
    trace_dir = os.path.join(output_dir, "trace")
    expect_path = os.path.join(output_dir, "expect")
    metadata_path = os.path.join(trace_dir, "metadata")
    data_path = os.path.join(trace_dir, "data")
    os.makedirs(trace_dir, exist_ok=True)

    with open(mp_path, "r") as f:
        parts = moultipart.parse(f)

    with open(metadata_path, "w") as f:
        f.write(_make_ctf_metadata(parts[0].content.strip()))

    with open(data_path, "wb") as f:
        f.write(_make_ctf_1_data(parts[1].content))

    with open(expect_path, "w") as f:
        f.write(parts[2].content)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "mp_path", metavar="MP-PATH", help="moultipart document to process"
    )
    parser.add_argument(
        "output_dir",
        metavar="OUTPUT-DIR",
        help="output directory for the CTF trace and expectation file",
    )
    args = parser.parse_args()
    _create_files_from_mp(args.mp_path, args.output_dir)



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2023 EfficiOS Inc.
#
# pyright: strict, reportTypeCommentUsage=false

import os
import sys
import typing
import argparse
from typing import Any, List, Union

import normand
import moultipart


class ErrorCause:
    def __init__(self, what: str, line_no: int, col_no: int):
        self._what = what
        self._line_no = line_no
        self._col_no = col_no

    @property
    def what(self):
        return self._what

    @property
    def line_no(self):
        return self._line_no

    @property
    def col_no(self):
        return self._col_no


class Error(RuntimeError):
    def __init__(self, causes: List[ErrorCause]):
        self._causes = causes

    @property
    def causes(self):
        return self._causes


def _write_file(
    name: str, base_dir: str, content: Union[str, bytearray], verbose: bool
):
    path = os.path.join(base_dir, name)

    if verbose:
        print("Writing `{}`.".format(os.path.normpath(path)))

    os.makedirs(os.path.normpath(os.path.dirname(path)), exist_ok=True)

    with open(path, "w" if isinstance(content, str) else "wb") as f:
        f.write(content)


def _normand_parse(
    part: moultipart.Part, init_vars: normand.VariablesT, init_labels: normand.LabelsT
):
    try:
        return normand.parse(
            part.content, init_variables=init_vars, init_labels=init_labels
        )
    except normand.ParseError as e:
        raise Error(
            [
                ErrorCause(
                    msg.text,
                    msg.text_location.line_no + part.first_content_line_no - 1,
                    msg.text_location.col_no,
                )
                for msg in e.messages
            ]
        ) from e


def _reformat_ctf_2_metadata(content: str):
    import json

    json_seq_fragments = []  # type: list[str]

    for fragment in json.loads(content):
        json_seq_fragments.append("\x1e{}".format(json.dumps(fragment, indent=2)))

    return "\n".join(json_seq_fragments)


def _reformat_metadata(content: str):
    content = content.strip() + "\n"

    if content.startswith("["):
        # CTF 2: JSON array to JSON text sequence
        return _reformat_ctf_2_metadata(content)

    return content


def _generate_from_part(
    part: moultipart.Part,
    base_dir: str,
    verbose: bool,
    normand_vars: normand.VariablesT,
    normand_labels: normand.LabelsT,
):
    content = part.content

    if part.header_info == "metadata":
        content = _reformat_metadata(content)
    else:
        res = _normand_parse(part, normand_vars, normand_labels)
        content = res.data
        normand_vars = res.variables
        normand_labels = res.labels

    _write_file(part.header_info, base_dir, content, verbose)
    return normand_vars, normand_labels


def generate(input_path: str, base_dir: str, verbose: bool):
    with open(input_path) as input_file:
        variables = {}  # type: normand.VariablesT
        labels = {}  # type: normand.LabelsT

        for part in moultipart.parse(input_file):
            variables, labels = _generate_from_part(
                part, base_dir, verbose, variables, labels
            )


def _parse_cli_args():
    argparser = argparse.ArgumentParser()
    argparser.add_argument(
        "input_path", metavar="PATH", type=str, help="moultipart input file name"
    )
    argparser.add_argument(
        "--base-dir", type=str, help="base directory of generated files", default=""
    )
    argparser.add_argument(
        "--verbose", "-v", action="store_true", help="increase verbosity"
    )
    return argparser.parse_args()


def _run_cli(args: Any):
    generate(
        typing.cast(str, args.input_path),
        typing.cast(str, args.base_dir),
        typing.cast(bool, args.verbose),
    )


def _try_run_cli():
    args = _parse_cli_args()

    try:
        _run_cli(args)
    except Error as exc:
        print("Failed to process Normand part:", file=sys.stderr)

        for cause in reversed(exc.causes):
            print(
                "  {}:{}:{} - {}{}".format(
                    os.path.abspath(args.input_path),
                    cause.line_no,
                    cause.col_no,
                    cause.what,
                    "." if cause.what[-1] not in ".:;" else "",
                ),
                file=sys.stderr,
            )

        sys.exit(1)


if __name__ == "__main__":
    _try_run_cli()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import bt2
from cli_params_to_string import to_string


@bt2.plugin_component_class
class SourceWithQueryThatPrintsParams(
    bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
):
    @classmethod
    def _user_query(cls, executor, obj, params, method_obj):
        if obj == "please-fail":
            raise ValueError("catastrophic failure")

        return obj + ":" + to_string(params)


bt2.register_plugin(__name__, "query")



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import os
import unittest

import bt2

_TEST_PLUGIN_PLUGINS_PATH = os.environ["BT_PLUGINS_PATH"]
_TEST_PLUGIN_PLUGIN_EXTENSION_BY_OS = {"cygwin": "dll", "mingw": "dll"}


class PluginSetTestCase(unittest.TestCase):
    def test_create(self):
        pset = bt2.find_plugins_in_path(_TEST_PLUGIN_PLUGINS_PATH)
        self.assertTrue(len(pset) >= 3)

    def test_getitem(self):
        pset = bt2.find_plugins_in_path(_TEST_PLUGIN_PLUGINS_PATH)
        self.assertTrue(pset[0].path.startswith(_TEST_PLUGIN_PLUGINS_PATH))

    def test_iter(self):
        pset = bt2.find_plugins_in_path(_TEST_PLUGIN_PLUGINS_PATH)
        names = set()

        for plugin in pset:
            names.add(plugin.name)

        self.assertTrue("ctf" in names)
        self.assertTrue("utils" in names)
        self.assertTrue("text" in names)


class FindPluginsTestCase(unittest.TestCase):
    def test_find_nonexistent_dir(self):
        with self.assertRaises(ValueError):
            bt2.find_plugins_in_path(
                "/this/does/not/exist/246703df-cb85-46d5-8406-5e8dc4a88b41"
            )

    def test_find_none_existing_dir(self):
        plugins = bt2.find_plugins_in_path(_TEST_PLUGIN_PLUGINS_PATH, recurse=False)
        self.assertIsNone(plugins)

    def test_find_dir(self):
        pset = bt2.find_plugins_in_path(_TEST_PLUGIN_PLUGINS_PATH)
        self.assertTrue(len(pset) >= 3)

    def test_find_file(self):
        extension = _TEST_PLUGIN_PLUGIN_EXTENSION_BY_OS.get(
            os.environ["BT_TESTS_OS_TYPE"], "so"
        )
        plugin_name = "babeltrace-plugin-utils.{}".format(extension)
        path = os.path.join(_TEST_PLUGIN_PLUGINS_PATH, "utils", ".libs", plugin_name)
        pset = bt2.find_plugins_in_path(path)
        self.assertTrue(len(pset) == 1)


class FindPluginTestCase(unittest.TestCase):
    def test_find_none(self):
        plugin = bt2.find_plugin(
            "this-does-not-exist-246703df-cb85-46d5-8406-5e8dc4a88b41"
        )
        self.assertIsNone(plugin)

    def test_find_existing(self):
        plugin = bt2.find_plugin("ctf", find_in_user_dir=False, find_in_sys_dir=False)
        self.assertIsNotNone(plugin)


class PluginTestCase(unittest.TestCase):
    def setUp(self):
        self._plugin = bt2.find_plugin(
            "ctf", find_in_user_dir=False, find_in_sys_dir=False
        )

    def tearDown(self):
        del self._plugin

    def test_name(self):
        self.assertEqual(self._plugin.name, "ctf")

    def test_path(self):
        plugin_path = os.path.abspath(os.path.normcase(self._plugin.path))
        plugin_path_env = os.path.abspath(os.path.normcase(_TEST_PLUGIN_PLUGINS_PATH))
        self.assertTrue(plugin_path.startswith(plugin_path_env))

    def test_author(self):
        self.assertEqual(self._plugin.author, "EfficiOS <https://www.efficios.com/>")

    def test_license(self):
        self.assertEqual(self._plugin.license, "MIT")

    def test_description(self):
        self.assertEqual(self._plugin.description, "CTF input and output")

    def test_version(self):
        self.assertIsNotNone(self._plugin.version)
        self.assertEqual(self._plugin.version.major, 2)
        self.assertEqual(self._plugin.version.minor, 0)
        self.assertEqual(self._plugin.version.patch, 0)
        self.assertIsNone(self._plugin.version.extra)

    def test_source_comp_classes_len(self):
        self.assertEqual(len(self._plugin.source_component_classes), 2)

    def test_source_comp_classes_getitem(self):
        self.assertEqual(self._plugin.source_component_classes["fs"].name, "fs")

    def test_source_comp_classes_getitem_invalid(self):
        with self.assertRaises(KeyError):
            self._plugin.source_component_classes["lol"]

    def test_source_comp_classes_iter(self):
        plugins = {}

        for cc_name, cc in self._plugin.source_component_classes.items():
            plugins[cc_name] = cc

        self.assertTrue("fs" in plugins)
        self.assertTrue("lttng-live" in plugins)
        self.assertEqual(plugins["fs"].name, "fs")
        self.assertEqual(plugins["lttng-live"].name, "lttng-live")

    def test_filter_comp_classes_len(self):
        plugin = bt2.find_plugin("utils", find_in_user_dir=False, find_in_sys_dir=False)
        self.assertEqual(len(plugin.filter_component_classes), 2)

    def test_sink_comp_classes_len(self):
        self.assertEqual(len(self._plugin.sink_component_classes), 1)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: MIT
#
# Copyright (c) 2016 Philippe Proulx <pproulx@efficios.com>
#

import sys
import argparse
import unittest

from tap import TAPTestRunner

if __name__ == "__main__":
    argparser = argparse.ArgumentParser()
    argparser.add_argument(
        "-f", "--failfast", help="Stop on first fail or error", action="store_true"
    )

    argparser.add_argument(
        "start_dir", help="Base directory where to search for tests", type=str
    )

    mut_exclu_group = argparser.add_mutually_exclusive_group(required=True)

    mut_exclu_group.add_argument(
        "-p",
        "--pattern",
        help="Glob-style pattern of test files to run " "(e.g. test_event*.py)",
        type=str,
    )

    mut_exclu_group.add_argument(
        "-t",
        "--test-case",
        help="Run a specific test module name, test class "
        "name, or test method name "
        "(e.g. test_event.EventTestCase.test_clock_value)",
        type=str,
    )

    args = argparser.parse_args()

    loader = unittest.TestLoader()

    start_dir = args.start_dir
    pattern = args.pattern
    failfast = args.failfast
    test_case = args.test_case

    if test_case:
        sys.path.append(start_dir)
        tests = loader.loadTestsFromName(test_case)
    elif pattern:
        tests = loader.discover(start_dir, pattern)
    else:
        # This will never happen because the mutual exclusion group has the
        # `required` parameter set to True. So one or the other must be set or
        # else it will fail to parse.
        sys.exit(1)

    if tests.countTestCases() < 1:
        print("No tests matching '%s' found in '%s'" % (pattern, start_dir))
        sys.exit(1)

    runner = TAPTestRunner(failfast=failfast)
    runner.set_stream(True)
    runner.set_format("{method_name}")
    sys.exit(0 if runner.run(tests).wasSuccessful() else 1)



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import unittest

import bt2
from bt2 import port as bt2_port
from bt2 import connection as bt2_connection


class ConnectionTestCase(unittest.TestCase):
    def test_create(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(self, config, params, obj):
                self._add_output_port("out")

        class MySink(bt2._UserSinkComponent):
            def __init__(self, config, params, obj):
                self._add_input_port("in")

            def _user_consume(self):
                raise bt2.Stop

        graph = bt2.Graph()
        src = graph.add_component(MySource, "src")
        sink = graph.add_component(MySink, "sink")
        conn = graph.connect_ports(src.output_ports["out"], sink.input_ports["in"])
        self.assertIs(type(conn), bt2_connection._ConnectionConst)

    def test_downstream_port(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(self, config, params, obj):
                self._add_output_port("out")

        class MySink(bt2._UserSinkComponent):
            def __init__(self, config, params, obj):
                self._add_input_port("in")

            def _user_consume(self):
                raise bt2.Stop

        graph = bt2.Graph()
        src = graph.add_component(MySource, "src")
        sink = graph.add_component(MySink, "sink")
        conn = graph.connect_ports(src.output_ports["out"], sink.input_ports["in"])
        self.assertEqual(conn.downstream_port.addr, sink.input_ports["in"].addr)
        self.assertIs(type(conn), bt2_connection._ConnectionConst)
        self.assertIs(type(conn.downstream_port), bt2_port._InputPortConst)

    def test_upstream_port(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(self, config, params, obj):
                self._add_output_port("out")

        class MySink(bt2._UserSinkComponent):
            def __init__(self, config, params, obj):
                self._add_input_port("in")

            def _user_consume(self):
                raise bt2.Stop

        graph = bt2.Graph()
        src = graph.add_component(MySource, "src")
        sink = graph.add_component(MySink, "sink")
        conn = graph.connect_ports(src.output_ports["out"], sink.input_ports["in"])
        self.assertEqual(conn.upstream_port.addr, src.output_ports["out"].addr)
        self.assertIs(type(conn.upstream_port), bt2_port._OutputPortConst)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import unittest

import utils
from bt2 import field as bt2_field
from bt2 import stream as bt2_stream
from utils import run_in_component_init


class PacketTestCase(unittest.TestCase):
    @staticmethod
    def _create_packet(with_pc):
        def create_tc_cc(comp_self):
            cc = comp_self._create_clock_class(frequency=1000, name="my_cc")
            tc = comp_self._create_trace_class()
            return cc, tc

        clock_class, tc = run_in_component_init(0, create_tc_cc)

        # packet context
        pc = (
            tc.create_structure_field_class(
                members=(
                    ("something", tc.create_signed_integer_field_class(8)),
                    ("something_else", tc.create_double_precision_real_field_class()),
                    ("events_discarded", tc.create_unsigned_integer_field_class(64)),
                    ("packet_seq_num", tc.create_unsigned_integer_field_class(64)),
                )
            )
            if with_pc
            else None
        )

        # stream
        stream = tc().create_stream(
            tc.create_stream_class(
                default_clock_class=clock_class,
                event_common_context_field_class=tc.create_structure_field_class(
                    members=(
                        ("cpu_id", tc.create_signed_integer_field_class(8)),
                        ("stuff", tc.create_double_precision_real_field_class()),
                    )
                ),
                packet_context_field_class=pc,
                supports_packets=True,
            )
        )

        # packet
        return stream.create_packet(), stream, pc

    def test_attr_stream(self):
        packet, stream, _ = self._create_packet(with_pc=True)
        self.assertEqual(packet.stream.addr, stream.addr)
        self.assertIs(type(packet.stream), bt2_stream._Stream)

    def test_const_attr_stream(self):
        packet = utils.get_const_packet_beginning_message().packet
        self.assertIs(type(packet.stream), bt2_stream._StreamConst)

    def test_context_field(self):
        packet, stream, pc_fc = self._create_packet(with_pc=True)
        self.assertEqual(packet.context_field.cls.addr, pc_fc.addr)
        self.assertIs(type(packet.context_field), bt2_field._StructureField)

    def test_const_context_field(self):
        packet = utils.get_const_packet_beginning_message().packet
        self.assertIs(type(packet.context_field), bt2_field._StructureFieldConst)

    def test_no_context_field(self):
        packet, _, _ = self._create_packet(with_pc=False)
        self.assertIsNone(packet.context_field)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 Simon Marchi <simon.marchi@efficios.com>
#


import unittest

import bt2

query_object = "metadata-info"


class QuerySupportInfoTestCase(unittest.TestCase):
    def setUp(self):
        ctf = bt2.find_plugin("ctf")
        assert ctf
        self._fs = ctf.source_component_classes["fs"]

    def test_non_map_params(self):
        with self.assertRaisesRegex(
            bt2._Error, "Error validating parameters: top-level is not a map value"
        ):
            bt2.QueryExecutor(self._fs, query_object).query()


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import bt2
from cli_params_to_string import to_string


@bt2.plugin_component_class
class SinkThatPrintsParams(bt2._UserSinkComponent):
    def __init__(self, config, params, obj):
        self._add_input_port("in")
        print(to_string(params))

    def _user_consume(self):
        raise bt2.Stop


bt2.register_plugin(__name__, "params")



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import unittest

import bt2


class _DummySink(bt2._UserSinkComponent):
    def _user_consume(self):
        pass


class ComponentDescriptorTestCase(unittest.TestCase):
    def setUp(self):
        self._obj = object()
        self._comp_descr = bt2.ComponentDescriptor(_DummySink, {"zoom": -23}, self._obj)

    def _get_comp_cls_from_plugin(self):
        plugin = bt2.find_plugin("text", find_in_user_dir=False, find_in_sys_dir=False)
        assert plugin is not None
        cc = plugin.source_component_classes["dmesg"]
        assert cc is not None
        return cc

    def test_init_invalid_cls_type(self):
        with self.assertRaises(TypeError):
            bt2.ComponentDescriptor(int)

    def test_init_invalid_params_type(self):
        with self.assertRaises(TypeError):
            bt2.ComponentDescriptor(_DummySink, object())

    def test_init_invalid_obj_non_python_comp_cls(self):
        cc = self._get_comp_cls_from_plugin()

        with self.assertRaises(ValueError):
            bt2.ComponentDescriptor(cc, obj=57)

    def test_init_with_user_comp_cls(self):
        bt2.ComponentDescriptor(_DummySink)

    def test_init_with_gen_comp_cls(self):
        cc = self._get_comp_cls_from_plugin()
        bt2.ComponentDescriptor(cc)

    def test_attr_component_class(self):
        self.assertIs(self._comp_descr.component_class, _DummySink)

    def test_attr_params(self):
        self.assertEqual(self._comp_descr.params, {"zoom": -23})

    def test_attr_obj(self):
        self.assertIs(self._comp_descr.obj, self._obj)


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2019 EfficiOS Inc.
#

import os

import bt2

# This file defines source component classes to help verify the parameters an
# log levels passed to components.  Each component creates one stream, with a
# name derived from either:
#
#   - the received params that start with `test-`
#   - the received log level
#
# The `what` parameter determines what is used.


class TestIter(bt2._UserMessageIterator):
    def __init__(self, config, output_port):
        params = output_port.user_data["params"]
        obj = output_port.user_data["obj"]

        comp_cls_name = self._component.__class__.__name__

        if params["what"] == "test-params":
            items = sorted([str(x) for x in params.items() if x[0].startswith("test-")])
            stream_name = "{}: {}".format(comp_cls_name, ", ".join(items))
        elif params["what"] == "log-level":
            log_level = self._component.logging_level
            stream_name = "{}: {}".format(comp_cls_name, log_level)
        elif params["what"] == "python-obj":
            assert type(obj) is str or obj is None
            stream_name = "{}: {}".format(comp_cls_name, obj)
        else:
            assert False

        sc = output_port.user_data["sc"]
        tc = sc.trace_class
        t = tc()
        s = t.create_stream(sc, name=stream_name)

        self._msgs = [
            self._create_stream_beginning_message(s),
            self._create_stream_end_message(s),
        ]

    def __next__(self):
        if len(self._msgs) == 0:
            raise StopIteration

        return self._msgs.pop(0)


class Base:
    def __init__(self, params, obj):
        tc = self._create_trace_class()
        sc = tc.create_stream_class()

        self._add_output_port("out", {"params": params, "obj": obj, "sc": sc})


@bt2.plugin_component_class
class TestSourceA(Base, bt2._UserSourceComponent, message_iterator_class=TestIter):
    def __init__(self, config, params, obj):
        super().__init__(params, obj)

    @staticmethod
    def _user_query(priv_query_exec, obj, params, method_obj):
        # Match files starting with 'aaa'.

        if obj == "babeltrace.support-info":
            if params["type"] != "file":
                return 0

            name = os.path.basename(str(params["input"]))

            if name.startswith("aaa"):
                return {"weight": 1, "group": "aaa"}
            else:
                return 0
        else:
            raise bt2.UnknownObject


@bt2.plugin_component_class
class TestSourceB(Base, bt2._UserSourceComponent, message_iterator_class=TestIter):
    def __init__(self, config, params, obj):
        super().__init__(params, obj)

    @staticmethod
    def _user_query(priv_query_exec, obj, params, method_obj):
        # Match files starting with 'bbb'.

        if obj == "babeltrace.support-info":
            if params["type"] != "file":
                return 0

            name = os.path.basename(str(params["input"]))

            if name.startswith("bbb"):
                return {"weight": 1, "group": "bbb"}
            else:
                return 0
        else:
            raise bt2.UnknownObject


bt2.register_plugin(module_name=__name__, name="test")
